Struct plotlars::HorizontalBarPlot
source ยท pub struct HorizontalBarPlot { /* private fields */ }๐Deprecated since 0.5.0:
HorizontalBarPlot will be removed in v0.6.0. Please use BarPlot instead.Expand description
A structure representing a horizontal bar plot.
Implementations
sourceimpl HorizontalBarPlot
impl HorizontalBarPlot
sourcepub fn builder<'__f0, '__f1, '__f2, '__f3>() -> HorizontalBarPlotBuilder<'__f0, '__f1, '__f2, '__f3>
pub fn builder<'__f0, '__f1, '__f2, '__f3>() -> HorizontalBarPlotBuilder<'__f0, '__f1, '__f2, '__f3>
Creates a new HorizontalBarPlot.
Arguments
data- A reference to theDataFramecontaining the data to be plotted.x- A string specifying the column name to be used for the x-axis.y- A string specifying the column name to be used for the y-axis.group- An optional string specifying the column name to be used for grouping data points.error- An optional string specifying the column name containing error values for the x-axis data.color- An optionalRgbvalue specifying the color of the markers to be used for the plot.colors- An optional vector ofRgbvalues specifying the colors to be used for the plot.plot_title- An optionalTextstruct specifying the title of the plot.x_title- An optionalTextstruct specifying the title of the x-axis.y_title- An optionalTextstruct specifying the title of the y-axis.legend_title- An optionalTextstruct specifying the title of the legend.x_axis- An optional reference to anAxisstruct for customizing the x-axis.y_axis- An optional reference to anAxisstruct for customizing the y-axis.legend- An optional reference to aLegendstruct for customizing the legend of the plot (e.g., positioning, font, etc.).
Returns
Returns an instance of HorizontalBarPlot.
Example
HorizontalBarPlot::builder()
.data(&dataset)
.x("values")
.y("animals")
.group("gender")
.error("errors")
.color(Rgb(255, 0, 0))
.plot_title(
Text::from("Horizontal Bar Plot")
.font("Arial")
.size(18)
)
.x_title(
Text::from("value")
.font("Arial")
.size(15)
)
.y_title(
Text::from("animal")
.font("Arial")
.size(15)
)
.legend_title(
Text::from("gender")
.font("Arial")
.size(15)
)
.build()
.plot();